// Copyright (c) 2021 NetEase, Inc.  All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:convert';

import 'package:nim_core/nim_core.dart';
import 'package:yunxin_integration_test/yunxin_integration_test.dart';
import 'method_name_value.dart';

class HandleLoginCase extends HandleBaseCase {
  HandleLoginCase();

  @override
  Future<ResultBean?> handle(event) async {
    super.handle(event);
    var handled = false;
    var result;
    switch (methodName) {
      case login:
        Map map = jsonDecode(params![0])['loginInfo'];
        result = await NimCore.instance.authService
            .login(NIMLoginInfo.fromMap(Map<String, dynamic>.from(map)));
        _addUserObserver();
        _addMessageObserver();
        handled = true;
        break;
      case logout:
        result = await NimCore.instance.authService.logout();
        handled = true;
        break;
    }
    if (handled) {
      print('AuthServiceTest#$methodName result: ${result.code}');
      handleCaseResult = ResultBean(
        code: result.code,
        data: result.toMap(),
        message: result.errorDetails,
      );
    }
    return handleCaseResult;
  }

  _addUserObserver() {
    print('Test_Observer addUserObserver');
    NimCore.instance.userService.onFriendAddedOrUpdated.listen((event) {
      print('Test_Observer onFriendAddedOrUpdated ${event.toString()}');
    });

    NimCore.instance.userService.onUserInfoChange.listen((event) {
      print('Test_Observer onUserInfoChange ${event.toString()}');
    });

    NimCore.instance.userService.onFriendDeleted.listen((event) {
      print('Test_Observer onFriendDeleted ${event.toString()}');
    });

    NimCore.instance.userService.onBlackListChanged.listen((event) {
      print('Test_Observer onBlackListChanged');
    });

    NimCore.instance.systemMessageService.onReceiveSystemMsg.listen((event) {
      print(
          'Test_Observer onReceiveSystemMsg ${event.type} ${event.fromAccount}');
      if (event.type == SystemMessageType.addFriend) {
        NimCore.instance.userService
            .ackAddFriend(event.fromAccount ?? '', true);
      }
    });
  }

  _addMessageObserver() {
    print('Test_Observer addMessageObserver');
    NimCore.instance.messageService.onMessageRevoked.listen((event) {
      print('Test_Observer onMessageRevoked ${event.toString()}');
    });

    NimCore.instance.messageService.onMessageReceipt.listen((event) {
      print('Test_Observer onMessageReceipt ${event.toString()}');
    });

    NimCore.instance.messageService.onTeamMessageReceipt.listen((event) {
      print('Test_Observer onTeamMessageReceipt ${event.toString()}');
    });

    NimCore.instance.messageService.onMessage.listen((event) {
      print('Test_Observer onMessage ${event.toString()}');
      for (NIMMessage m in event) {
        if (m.sessionType == NIMSessionType.p2p) {
          NimCore.instance.messageService.sendMessageReceipt(sessionId: m.fromAccount!, message: m);
        } else if (m.sessionType == NIMSessionType.team) {
          NimCore.instance.messageService.sendTeamMessageReceipt(m);
        }
      }
    });
  }
}
